-- Got tired of looking up delay times so i hacked this up
-- // Garf //

local BPMtool_name = "Delay Times";

local delay_preferences = renoise.Document.create("DelayTimesPreferences") {

  show_decimals = false,
  
  show_2 = false,
  show_4 = true,
  show_8 = true,
  show_16 = true,
  show_32 = true,
  show_64 = true,
  show_128 = true,
  show_256 = false,
  
  show_2_dotted = false,
  show_4_dotted = true,
  show_8_dotted = true,
  show_16_dotted = false,
  show_32_dotted = false,
  show_64_dotted = false,
  show_128_dotted = false,
  show_256_dotted = false,

  show_2_triplets = false,
  show_4_triplets = true,
  show_8_triplets = true,
  show_16_triplets = false,
  show_32_triplets = false,
  show_64_triplets = false,
  show_128_triplets = false,
  show_256_triplets = false,
  
}

renoise.tool().preferences = delay_preferences

--------------------------------------------------------------------------------

-- Functions
function createDelayRow(item)

  local vb = renoise.ViewBuilder()

  return vb:horizontal_aligner {
      width = "100%",
      
      vb:text { 
        text = "   1/"..item, 
        width = "16%",
        font = "bold",
        align = "left",
        style = "strong"
      },
      
      -- DOTTED
      vb:checkbox { 
        id = "show_"..item.."_dotted",
        width = "28%",
        value = delay_preferences["show_"..item.."_dotted"].value,
        notifier = function(value)
                    delay_preferences["show_"..item.."_dotted"].value = value
                   end,
      },
      
      -- NORMAL
      vb:checkbox { 
        id = "show_"..item.."_normal",
        width = "28%",
        value = delay_preferences["show_"..item].value,
        notifier = function(value)
                    delay_preferences["show_"..item].value = value
                   end,
      },
      
      -- TRIPLET
      vb:checkbox { 
        id = "show_"..item.."_triplet",
        width = "28%",
        value = delay_preferences["show_"..item.."_triplets"].value,
        notifier = function(value)
                    delay_preferences["show_"..item.."_triplets"].value = value
                   end,
      },
  }
  
end


-- GUI
function delayTimesPreferences()

  local vb = renoise.ViewBuilder()

  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local CONTENT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING
  
  
  -- Introduction text
  local introduction = vb:column {
  
    vb:column {
      style = "group",
      margin = CONTENT_SPACING,

      vb:text {
        text = "Select which delay times you want shown in the status bar below\n"..
         "and then use the “show Delay Times in status bar” key binding", 
        width = 310,
        align = "center"
      },
    }
  }
  
  
  -- Header row
  local delay_table_header_row = vb:horizontal_aligner {
      width = "100%",
      
      vb:text { 
        text = "", 
        width = "16%"
      },
      
      vb:text { 
        text = "DOTTED", 
        width = "28%",
        align = "center"
      },
      
      vb:text { 
        text = "NORMAL", 
        width = "28%",
        align = "center"
      },
      
      vb:text { 
        text = "TRIPLET", 
        width = "28%",
        align = "center"
      },
    
  }
  
  
  -- Show Decimals or rounding number
  local showdecimal_row = vb:row {
  
    vb:text { 
      text = "Show decimals in status bar"
    },
    vb:checkbox { 
      id = "showdecimals",
      tooltip = "Show decimals instead of rounding the number",
      value = delay_preferences.show_decimals.value,
      notifier = function(value)
                  delay_preferences.show_decimals.value = value
                 end,
    },
    vb:text { 
      text = "    "
    },
    vb:button {
      text = "SHOW STATUS BAR",
      notifier = function()
        showDelayTimes()
      end
    }
    
  }
  
  
  -- Show Screen
  renoise.app():show_custom_dialog(
    "Delay Times - preferences", 
    
    vb:column {
      margin = DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      uniform = true,
  
      vb:column {
        spacing = CONTENT_SPACING,
        
        introduction,
        -- BPM_row,
        vb:space {
          height = 5
        },
        delay_table_header_row,
        createDelayRow(2),
        createDelayRow(4),
        createDelayRow(8),
        createDelayRow(16),
        createDelayRow(32),
        createDelayRow(64),
        createDelayRow(128),
        createDelayRow(256),
        vb:space {
          height = 15
        },
        showdecimal_row
      }
    }
  )
  
end



function formatDelayTimes(beat, factor, type)

  local val = beat

  -- Show decimals or rounded number
  if (delay_preferences.show_decimals.value) then
    local delaytime =  string.format("%.2f",val)
    -- strip cases where the number ends with .00
    return string.gsub(delaytime, "%.00", "")
  else
    return math.floor(val+0.5);
  end

end



function showDelayTimes()

  local bpm = renoise.song().transport.bpm;
  local beat = 60000/bpm;
  
  -- Create output string
  local output = "Delay Times ("..bpm.." BPM):  ";
  
  local temp = 1;
  local multiplier = 2;
  
  for i = 1,8,1 
  do 
     temp = temp*2
     
    if (delay_preferences["show_"..temp.."_dotted"].value) then
      output = output.."  1/"..temp.."d: ".. formatDelayTimes(beat*(multiplier*1.5)).."ms  ";
    end  
    
    if (delay_preferences["show_"..temp].value) then
      output = output.."  1/"..temp..": ".. formatDelayTimes(beat*multiplier).."ms  ";
    end  
    
    if (delay_preferences["show_"..temp.."_triplets"].value) then
      output = output.."  1/"..temp.."t: ".. formatDelayTimes(beat*(multiplier*0.6666667)).."ms  ";
    end
    
     multiplier = multiplier/2
  end
  
  renoise.app():show_status(output);
    
end


-- notifier callbacks
delay_preferences.show_decimals:add_notifier(showDelayTimes)



-- Key bindings
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:"..BPMtool_name.."...",
  invoke = delayTimesPreferences
}

renoise.tool():add_keybinding {
  name = "Global:Tools:show Delay Times in status bar",
  invoke =  showDelayTimes
}

renoise.tool():add_keybinding {
  name = "Global:Tools:show Delay Times preferences",
  invoke =  delayTimesPreferences
}

